module Test.Offline.Ngrams (tests) where

import Prelude

import Data.Text qualified as T
import Gargantext.API.Ngrams.Types qualified as NT
import Gargantext.Core
import Gargantext.Core.Types
import Gargantext.Database.Action.Flow.Utils (docNgrams)
import Gargantext.Database.Admin.Types.Hyperdata
import Test.Instances ()
import Test.QuickCheck
import Test.Tasty
import Test.Tasty.QuickCheck (testProperty)

-- In order to test the behaviour of 'docNgrams' we create wrappers around 'NgramsTerm' to have two
-- different 'Arbitrary' flavours, one that always produces non-empty 'Text' fragments, and one that
-- /might/ occasionally generate empty text fragments.
newtype NgramsTermNonEmpty = NgramsTermNonEmpty { unNgramsTermNonEmpty :: T.Text }
  deriving (Eq, Show)

instance Arbitrary NgramsTermNonEmpty where
  arbitrary = do
    singleChar <- arbitrary `suchThat` ((/=) ' ')
    txt        <- arbitrary
    pure $ NgramsTermNonEmpty $ (T.singleton singleChar <> txt)

tests :: TestTree
tests = testGroup "Ngrams" [
    testGroup "docNgrams" [
     testProperty "return results for non-empty input ngrams" testDocNgramsWithTerms
    ]
  ]

testDocNgramsWithTerms :: Lang -> NonEmptyList NgramsTermNonEmpty -> ContextOnlyId HyperdataDocument -> Property
testDocNgramsWithTerms lang ts doc =
  let ts' = map (NT.NgramsTerm . unNgramsTermNonEmpty) $ getNonEmpty ts
  in counterexample "docNgrams returned no results" $ length (docNgrams lang ts' doc) > 0
